'' +--------------------------------------------------------------------------+
'' | Cluso's RamBlade & TriBladeProp:  PropBootSD (boot from SD card)   v1.10 |
'' +--------------------------------------------------------------------------+
'' |  Author:            "Cluso99" (Ray Rodrick)                              |
'' |  Copyright (c) 2010 "Cluso99" (Ray Rodrick)                              |
'' |  License            MIT License - See end of file for terms of use       |
'' +--------------------------------------------------------------------------+
                             
'' Thanks to Jeff Ledger for PropDOS, Mike Green & Tomas Rokicki for modified Fsrw code (from FemtoBASIC)

'' This is a minimal eeprom boot code to automatically boot a Propeller Binary
''
'' RR20100108   PropBootSD_V1_00  minimal boot version (fixed DO tristate when booting)
'' RR20120120   PropBootSD_V1_10  mod for RamBlade3
'' RR20120814   Strip to simplify as an example for Prop II

CON

'RamBlade3
  _clkmode  = xtal1 + pll16x
  _xinfreq  = 6_500_000
  spiDO     = 20                'microSD
  spiClk    = 21
  spiDI     = 22
  spiCS     = 23
  SIpin     = 31                'serial
  SOpin     = 30
  baud      = 115200

      
OBJ
     fsrw  : "fsrwsdspiFemto_rr005d"         'modified SD routines
     uart  : "FullDuplexSerial_rr004"
      
VAR
     long   ioControl[2]
     byte   buff[512]
          
PUB mainProgram | r, fbase, fsize

  uart.start(SIpin,SOpin,0,baud)
  waitcnt(clkfreq * 3 + cnt)
  uart.str(string(13,10))                               'crlf
  uart.str(string("RamBlade3 SD test loader for P2 V1.15"))
  uart.str(string(13,10))                               'crlf

'** start the SD card drivers **
  r := fsrw.start(@ioControl)                           'start I2C/SPI driver
  r := fsrw.mount(spiDO,spiClk,spiDI,spiCS)             'check for SD card, abort if not found.

  if r < 0
    uart.str(string("** SD card not found **"))
    failure                                             'failure routine

  r := fsrw.readSDCard(0, @buff, 512)                   'read the MBR sector
  'Validate the MBR sector identification string here <------
  fbase := buff[495] << 24 | buff[494] << 16 | buff[493] << 8 | buff[492]       'get the sector# for booting
    
  uart.str(string("Booting file... at sector = "))
  uart.dec(fbase)
  uart.str(string(" $"))
  uart.hex(fbase,8)
  uart.str(string(13,10))                               'crlf
  waitcnt(clkfreq / 2 + cnt)                              
  uart.stop
  fsrw.bootSDCard(fbase, 32768)                         'boot the prop passing sector# and size (likely to be 1024)

' ** here on failure **
  uart.str(string("** Boot failed **"))
  failure                                               'failure routine

PRI ValidateMBR
'will need to validate the following (subset??) identification string
  'update the MBR sector
  buff[480] := "P"                                      '\\ setup 12 byte identification string at $1E0
  buff[481] := "r"                                      '||   "PropellerII" +$00
  buff[482] := "o"                                      '||
  buff[483] := "p"                                      '||
  buff[484] := "e"                                      '||
  buff[485] := "l"                                      '||
  buff[486] := "l"                                      '||
  buff[487] := "e"                                      '||
  buff[488] := "r"                                      '||
  buff[489] := "I"                                      '||
  buff[490] := "I"                                      '||
  buff[491] := 0                                        '//


Pri Failure | t
'***** here only if it fails *****
  uart.str(string(13,10))                               'crlf
  waitcnt(clkfreq * 5 + cnt)                            
  uart.stop
  repeat t from 0 to 7                                  'Stop all COGs except this one
    if (t <> cogid)
      cogstop(t)
  cogstop(cogid)                                        'then stop this cog. All done!


dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}} 